@@ -4,9 +4,9 @@ use Moo 1.000007;
 use Carp;
 use Try::Tiny 0.11;
 use File::Spec 3.33;
-use DataStore::CAS 0.02;
+use DataStore::CAS 0.01;
 
-our $VERSION= '0.011000';
+our $VERSION= '0.0100';
 
 require DataStore::CAS::FS::Dir;
 require DataStore::CAS::FS::DirCodec::Universal;
@@ -25,23 +25,21 @@ has hash_of_empty_dir => ( is => 'lazy' );
 
 has dir_cache         => ( is => 'rw', default => sub { DataStore::CAS::FS::DirCache->new() } );
 
-# _nodes is a tree of nodes, each of the form:
+# _path_overrides is a tree of nodes, each of the form:
 # $node= {
-#   dirent  => $Dir_Entry,  # mandatory
+#   entry   => $Dir_Entry,  # mandatory
 #   dir     => $CAS_FS_Dir, # optional, created on demand
 #   subtree => {
 #     KEY1 => $node1,
 #     KEY2 => $node2,
 #     ...
 #   }
-#   changed => 1 # set if a path override has happened here, or in any child node
-#   invalid => 1 # set whenever a path override deletes this node
 # }
 #
 #  If 'case_insensitive' is true, the keys will all be upper-case, but the $Dir_Entry
 #  objects will contain the correct-case name.
 #
-has _nodes   => ( is => 'rw' );
+has _path_overrides   => ( is => 'rw' );
 
 
 sub _build_hash_of_empty_dir {
@@ -142,40 +140,21 @@ sub validate   { (shift)->store->validate(@_) }
 
 
 sub path {
-	bless { filesystem => (shift), path_names => [ map { File::Spec->splitdir($_) } @_ ] },
+	bless { filesystem => (shift), path_names => [ @_ ] },
 		'DataStore::CAS::FS::Path';
 }
 
 
-sub path_if_exists {
-	my $self= shift;
-	my $path= $self->path(@_);
-	$path->resolve({no_die => 1})? $path : undef;
-}
-
-
-sub tree_iterator {
-	my $self= shift;
-	my %p= (@_ == 1 && ref $_[0] eq 'HASH')? %{$_[0]} : @_;
-	return DataStore::CAS::FS::TreeIterator->new(
-		path => [],
-		%p,
-		fs => $self
-	);
-}
-
-
-
 sub resolve_path {
 	my ($self, $path, $flags)= @_;
 	$flags ||= {};
 	
-	my $ret= $self->_resolve_path(undef, $path, { follow_symlinks => 1, %$flags });
+	my $ret= $self->_resolve_path($path, { follow_symlinks => 1, %$flags });
 	
 	# Array means success, scalar means error.
 	if (ref($ret) eq 'ARRAY') {
 		# The user wants directory entries, not "nodes".
-		$_= $_->{dirent} for @$ret;
+		$_= $_->{entry} for @$ret;
 		return $ret;
 	}
 
@@ -187,24 +166,20 @@ sub resolve_path {
 }
 
 sub _resolve_path {
-	my ($self, $nodes, $path_names, $flags)= @_;
+	my ($self, $path, $flags)= @_;
 
-	my @path= ref($path_names)? @$path_names : File::Spec->splitdir($path_names);
-	$nodes ||= [];
-	push @$nodes, ($self->{_nodes} ||= { dirent => $self->root_entry })
-		unless @$nodes;
+	my @path= ref($path)? @$path : File::Spec->splitdir($path);
+	my @nodes= ( $self->_path_overrides || { entry => $self->root_entry } );
 	
-	my $mkdir_defaults;
-	sub _build_mkdir_defaults {
-		my $flags= shift;
-		my @ret= %{$flags->{mkdir_defaults}}
-			if defined $flags->{mkdir_defaults};
-		push @ret, type => 'dir', ref => undef;
-		\@ret
-	}
+	return "Root directory must be a directory"
+		unless $nodes[0]{entry}->type eq 'dir';
+
+	my @mkdir_defaults= %{$flags->{mkdir_defaults}}
+		if ref $flags->{mkdir_defaults};
+	push @mkdir_defaults, type => 'dir', ref => undef;
 
 	while (@path) {
-		my $ent= $nodes->[-1]{dirent};
+		my $ent= $nodes[-1]{entry};
 		my $dir;
 
 		# Support for "symlink" is always UNIX-based (or compatible)
@@ -217,10 +192,10 @@ sub _resolve_path {
 				or return 'Invalid symbolic link "'.$ent->name.'"';
 
 			unshift @path, split('/', $target, -1);
-			pop @$nodes;
+			pop @nodes;
 			
 			# If an absolute link, we start over from the root
-			@$nodes= ( $nodes->[0] )
+			@nodes= ( $nodes[0] )
 				if $path[0] eq '';
 
 			next;
@@ -230,7 +205,7 @@ sub _resolve_path {
 			return 'Cannot descend into directory entry "'.$ent->name.'" of type "'.$ent->type.'"'
 				unless ($flags->{mkdir}||0) > 1;
 			# Here, mkdir flag converts entry into a directory
-			$nodes->[-1]{dirent}= $ent->clone(@{ $mkdir_defaults ||= _build_mkdir_defaults($flags)});
+			$nodes[-1]{entry}= $ent->clone(@mkdir_defaults);
 		}
 
 		# Get the next path component, ignoring empty and '.'
@@ -241,31 +216,31 @@ sub _resolve_path {
 		# This is the same way the kernel does it, but perhaps shell behavior is preferred...
 		if ($name eq '..') {
 			return "Cannot access '..' at root directory"
-				unless @$nodes > 1;
-			pop @$nodes;
+				unless @nodes > 1;
+			pop @nodes;
 			next;
 		}
 
 		# If this directory has an in-memory override for this name, use it
 		my $subnode;
-		if ($nodes->[-1]{subtree}) {
+		if ($nodes[-1]{subtree}) {
 			my $key= $self->case_insensitive? uc $name : $name;
-			$subnode= $nodes->[-1]{subtree}{$key};
+			$subnode= $nodes[-1]{subtree}{$key};
 		}
+		if (!defined $subnode) {
+			# Else we need to find the name within the current directory
 
-		# Else we need to find the name within the current directory
-		if (!defined $subnode && (defined $nodes->[-1]{dir} || defined $ent->ref)) {
 			# load it if it isn't cached
-			($nodes->[-1]{dir} ||= $self->get_dir($ent->ref))
-				or return 'Failed to open directory "'.$ent->name.' ('.$ent->ref.')"';
-
-			# See if the directory contains this entry
-			if (defined (my $subent= $nodes->[-1]{dir}->get_entry($name))) {
-				$subnode= { dirent => $subent };
-				my $key= $self->case_insensitive? uc $name : $name;
-				# Weak reference, until _apply_overrides is called.
-				Scalar::Util::weaken( $nodes->[-1]{subtree}{$key}= $subnode );
+			if (!defined $nodes[-1]{dir} && defined $ent->ref) {
+				defined ( $nodes[-1]{dir}= $self->get_dir($ent->ref) )
+					or return 'Failed to open directory "'.$ent->name.' ('.$ent->ref.')"';
 			}
+
+			# If we're working on an available directory, try loading it
+			my $subent= $nodes[-1]{dir}->get_entry($name)
+				if defined $nodes[-1]{dir};
+			$subnode= { entry => $subent }
+				if defined $subent;
 		}
 
 		# If we haven't found one, or if it is 0 (deleted), either create or die.
@@ -273,152 +248,67 @@ sub _resolve_path {
 			# If we're supposed to create virtual entries, do so
 			if ($flags->{mkdir} or $flags->{partial}) {
 				$subnode= {
-					invalid => 1, # not valid until _apply_overrides
-					dirent => DataStore::CAS::FS::DirEnt->new(
+					entry => DataStore::CAS::FS::DirEnt->new(
 						name => $name,
 						# It is a directory if there are more path components to resolve.
-						(@path? @{ $mkdir_defaults ||= _build_mkdir_defaults($flags)} : ())
+						(@path? @mkdir_defaults : ())
 					)
 				};
 			}
 			# Else it doesn't exist and we fail.
 			else {
-				my $dir_path= File::Spec->catdir(map { $_->{dirent}->name } @$nodes);
+				my $dir_path= File::Spec->catdir(map { $_->{entry}->name } @nodes);
 				return "Directory \"$dir_path\" is not present in storage"
-					unless defined $nodes->[-1]{dir};
+					unless defined $nodes[-1]{dir};
 				return "No such directory entry \"$name\" at \"$dir_path\"";
 			}
 		}
 
-		push @$nodes, $subnode;
+		push @nodes, $subnode;
 	}
 	
-	$nodes;
-}
-
-
-sub get_dir_entries {
-	my ($self, $path)= @_;
-	my $nodes= $self->_resolve_path(undef, $path);
-	ref $nodes
-		or croak $nodes;
-	return $self->_get_dir_entries($nodes->[-1]);
-}
-
-sub readdir {
-	my $self= shift;
-	my @names= map { $_->name } @{ $self->get_dir_entries(@_) };
-	return wantarray? @names : \@names;
-}
-
-# This method combines the original directory with its overrides.
-sub _get_dir_entries {
-	my ($self, $node)= @_;
-	my $ent= $node->{dirent};
-	croak "Can't get listing for non-directory"
-		unless $ent->type eq 'dir';
-	my %dirents;
-	# load dir if it isn't cached
-	if (!defined $node->{dir} && defined $ent->ref) {
-		defined ( $node->{dir}= $self->get_dir($ent->ref) )
-			or return 'Failed to open directory "'.$ent->name.' ('.$ent->ref.')"';
-	}
-	my $caseless= $self->case_insensitive;
-	if (defined $node->{dir}) {
-		my $iter= $node->{dir}->iterator;
-		my $dirent;
-		$dirents{$caseless? uc($dirent->name) : $dirent->name}= $dirent
-			while defined ($dirent= $iter->());
-	}
-	if (my $t= $node->{subtree}) {
-		for (keys %$t) {
-			my $subnode= $t->{$_};
-			next unless defined $subnode;
-			die "BUG" if ref $subnode && $subnode->{invalid};
-			if (ref $subnode) {
-				$dirents{$_}= $subnode->{dirent}
-					if $subnode->{changed};
-			} else {
-				delete $dirents{$_};
-			}
-		}
-	}
-	return [ map { $dirents{$_} } sort keys %dirents ];
+	\@nodes;
 }
 
 
 sub set_path {
 	my ($self, $path, $newent, $flags)= @_;
 	$flags ||= {};
-	my $nodes= $self->_resolve_path(undef, $path, { follow_symlinks => 1, partial => 1, %$flags });
+	my $nodes= $self->_resolve_path($path, { follow_symlinks => 1, partial => 1, %$flags });
 	croak $nodes unless ref $nodes;
+
 	# replace the final entry, after applying defaults
 	if (!$newent) {
-		# unlink request.  Ignore if node didn't exist.
-		return if $nodes->[-1]{invalid};
-
-		# Can't unlink the root node
-		croak "Can't unlink root node"
-			unless @$nodes > 1;
-
-		$nodes->[-1]{invalid}= 1;
-		# Recursively invalidate all nodes beneath this one
-		&_invalidate_subtree for ($nodes->[-1]);
-
-		# Mark in prev node that this item is gone
-		my $key= $self->case_insensitive? uc $nodes->[-1]{dirent}->name : $nodes->[-1]{dirent}->name;
-		pop @$nodes;
-		$nodes->[-1]{subtree}{$key}= 0;
-	} else {
-		if (ref $newent eq 'HASH' or !defined $newent->name or !defined $newent->type) {
-			my %ent_hash= %{ref $newent eq 'HASH'? $newent : $newent->as_hash};
-			$ent_hash{name}= $nodes->[-1]{dirent}->name
-				unless defined $ent_hash{name};
-			defined $ent_hash{name} && length $ent_hash{name}
-				or die "No name for new dir entry";
-			$ent_hash{type}= $nodes->[-1]{dirent}->type || 'file'
-				unless defined $ent_hash{type};
-			$newent= DataStore::CAS::FS::DirEnt->new(\%ent_hash);
-		}
-		$nodes->[-1]{dirent}= $newent;
-		delete $nodes->[-1]{dir};
-		# Recursively invalidate all nodes beneath this one
-		&_invalidate_subtree for ($nodes->[-1]);
+		$newent= 0; # 0 means unlink
+	} elsif (ref $newent eq 'HASH' or !defined $newent->name or !defined $newent->type) {
+		my %ent_hash= %{ref $newent eq 'HASH'? $newent : $newent->as_hash};
+		$ent_hash{name}= $nodes->[-1]{entry}->name
+			unless defined $ent_hash{name};
+		defined $ent_hash{name} && length $ent_hash{name}
+			or die "No name for new dir entry";
+		$ent_hash{type}= $nodes->[-1]{entry}->type || 'file'
+			unless defined $ent_hash{type};
+		$newent= DataStore::CAS::FS::DirEnt->new(\%ent_hash);
 	}
-	# Now connect nodes with strong references, and mark as changed
+	$nodes->[-1]{entry}= $newent;
 	$self->_apply_overrides($nodes);
 }
-sub _invalidate_subtree {
-	if ($_->{subtree}) {
-		++$_->{invalid} && &_invalidate_subtree for grep { ref $_ } values %{delete $_->{subtree}};
-	}
-}
 
 
 sub update_path {
 	my ($self, $path, $changes, $flags)= @_;
 	$flags ||= {};
-	my $nodes= $self->_resolve_path(undef, $path, { follow_symlinks => 1, partial => 1, %$flags });
+	my $nodes= $self->_resolve_path($path, { follow_symlinks => 1, partial => 1, %$flags });
 	croak $nodes unless ref $nodes;
 
 	# update the final entry, after applying defaults
-	my $entref= \$nodes->[-1]{dirent};
-	my $old_dir_ref= defined $$entref->type && $$entref->type eq 'dir'? $$entref->ref : undef;
+	my $entref= \$nodes->[-1]{entry};
 	$$entref= $$entref->clone(
-		(defined $$entref->type? () : ( type => 'file' )),
+		defined $$entref->type? () : ( type => 'file' ),
 		ref $changes eq 'HASH'? %$changes
 			: ref $changes eq 'ARRAY'? @$changes
 			: croak 'parameter "changes" must be a hashref or arrayref'
 	);
-	my $new_dir_ref= $$entref->type eq 'dir'? $$entref->ref : undef;
-
-	# If we changed the type of a directory, or changed which digest_hash it
-	# refers to, then we should clear the subtree under this node.
-	if (($old_dir_ref || '') ne ($new_dir_ref || '') && $nodes->[-1]{subtree}) {
-		# Recursively invalidate all nodes beneath this one
-		&_invalidate_subtree for ($nodes->[-1]);
-		delete $nodes->[-1]{dir};
-	}
 
 	$self->_apply_overrides($nodes);
 }
@@ -426,33 +316,24 @@ sub update_path {
 sub _apply_overrides {
 	my ($self, $nodes)= @_;
 	# Ensure that each node is connected to the previous via 'subtree'.
-	# When we find the first changed node, we assume the rest are connected.
-	my $prev;
-	for (reverse @$nodes) {
-		if ($prev) {
-			my $key= $self->case_insensitive? uc($prev->{dirent}->name) : $prev->{dirent}->name;
-			$_->{subtree}{$key}= $prev;
-		}
-		last if $_->{changed} && !$_->{invalid};
-		delete $_->{invalid};
-		$_->{changed}= 1;
-		$prev= $_;
+	# When we find the first connected node, we assume the rest are connected.
+	my $i;
+	for ($i= $#$nodes; $i > 0; $i--) {
+		my $key= $self->case_insensitive? uc $nodes->[$i]{entry}->name : $nodes->[$i]{entry}->name;
+		my $childref= \$nodes->[$i-1]{subtree}{$key};
+		last if $$childref and $$childref eq $nodes->[$i];
+		$$childref= $nodes->[$i];
 	}
 	# Finally, make sure the root override is set
-	$self->{_nodes}= $nodes->[0];
+	$self->{_path_overrides}= $nodes->[0]
+		unless $i;
 	1;
 }
 
 
 sub mkdir {
 	my ($self, $path)= @_;
-	my $nodes= $self->_resolve_path(undef, $path, { follow_symlinks => 1, mkdir => 1 });
-	croak $nodes unless ref $nodes;
-	unless (defined $nodes->[-1]{dirent}->type) {
-		$nodes->[-1]{dirent}= $nodes->[-1]{dirent}->clone(type => 'dir');
-		$self->_apply_overrides($nodes);
-	}
-	1;
+	$self->set_path($path, { type => 'dir', ref => $self->hash_of_empty_dir });
 }
 
 
@@ -473,23 +354,20 @@ sub unlink {
 
 sub rollback {
 	my $self= shift;
-	if ($self->{_nodes} && $self->{_nodes}{changed}) {
-		&invalidate_node for ($self->{_nodes});
-		$self->{_nodes}= undef;
-	}
+	$self->{_path_overrides}= undef;
 	1;
 }
 
 
 sub commit {
 	my $self= shift;
-	if ($self->_nodes && $self->_nodes->{changed}) {
-		my $root_node= $self->_nodes;
+	if ($self->_path_overrides) {
+		my $root_node= $self->_path_overrides;
 		croak "Root override must be a directory"
-			unless $root_node->{dirent}->type eq 'dir';
+			unless $root_node->{entry}->type eq 'dir';
 		my $hash= $self->_commit_recursive($root_node);
-		$self->{root_entry}= $root_node->{dirent}->clone(ref => $hash);
-		$self->{_nodes}= undef;
+		$self->{root_entry}= $root_node->{entry}->clone(ref => $hash);
+		$self->{_path_overrides}= undef;
 	}
 	1;
 }
@@ -500,46 +378,35 @@ sub commit {
 sub _commit_recursive {
 	my ($self, $node)= @_;
 
-	my %changes;
+	my $subtree= $node->{subtree} || {};
 	my @entries;
-	if (my $subtree= $node->{subtree}) {
-		while (my ($k, $v)= each %{$node->{subtree}}) {
-			$changes{$k}= $v
-				if defined $v && ($v eq 0 || $v->{changed});
-		}
-	}
-	
-	# If no changes, return original directory (if it exists)
-	return $node->{dirent}->ref
-		if !%changes && defined $node->{dirent}->ref;
-	
+
 	# Walk the directory entries and filter out any that have been overridden.
-	if (defined $node->{dir} || defined $node->{dirent}->ref) {
-		($node->{dir} ||= $self->get_dir($node->{dirent}->ref))
-			or croak 'Failed to open directory "'.$node->{dirent}->name.' ('.$node->{dirent}->ref.')"';
+	if (defined $node->{dir} || defined $node->{entry}->ref) {
+		($node->{dir} ||= $self->get_dir($node->{entry}->ref))
+			or croak 'Failed to open directory "'.$node->{entry}->name.' ('.$node->{entry}->ref.')"';
 		
 		my ($iter, $ent);
-		my $caseless= $self->case_insensitive;
 		for ($iter= $node->{dir}->iterator; defined ($ent= $iter->()); ) {
-			push @entries, $ent unless $changes{$caseless? uc($ent->name) : $ent->name};
+			my $key= $self->case_insensitive? uc $ent->name : $ent->name;
+			push @entries, $ent
+				unless defined $subtree->{$key};
 		}
 	}
 
 	# Now append the modified entries.
 	# Skip the "0"s, which represent files to unlink.
-	for (grep { ref $_ } values %changes) {
+	for (grep { ref $_ } values %$subtree) {
 		# Check if node is a dir and needs committed
-		if ($_->{subtree} and $_->{dirent}->type eq 'dir' and $_->{changed}) {
+		if ($_->{subtree} and $_->{entry}->type eq 'dir') {
 			my $hash= $self->_commit_recursive($_);
-			$_->{dirent}= $_->{dirent}->clone( ref => $hash );
+			$_->{entry}= $_->{entry}->clone( ref => $hash );
+			delete $_->{subtree};
+			delete $_->{dir};
 		}
 		
-		push @entries, $_->{dirent};
+		push @entries, $_->{entry};
 	}
-
-	# Invalidate all children of this node
-	&_invalidate_subtree for ($node);
-	
 	# Now re-encode the directory, using the same type as orig_dir
 	return $self->hash_of_empty_dir
 		unless @entries;
@@ -552,208 +419,45 @@ sub _commit_recursive {
 package DataStore::CAS::FS::Path;
 use strict;
 use warnings;
+use Carp;
 
 
 # main attributes
-sub path_names       { $_[0]{path_names} }
-sub filesystem       { $_[0]{filesystem} }
-#sub _node_path       { $_[0]{_node_path} }
+sub path_names     { $_[0]{path_names} }
+sub path_ents      { $_[0]{path_ents} || $_[0]->resolve }
+sub filesystem     { $_[0]{filesystem} }
 
 # convenience accessors
-sub path_name_list   { @{$_[0]->path_names} }
-sub path_dirent_list { map { $_->{dirent} } @{$_[0]->resolve} }
-sub path_dirents     { [ $_[0]->path_dirent_list ] }
-sub dirent           { $_[0]->resolve->[-1]{dirent} }
-sub type             { $_[0]->resolve->[-1]{dirent}->type }
-sub name             { $_[0]->resolve->[-1]{dirent}->name }
-sub depth            { -1 + @{$_[0]->resolve} }
-
-
-sub canonical_path {
-	my $self= shift;
-	$self->{canonical_path} ||= do {
-		my $name= $self->path_names;
-		my $path= '/'.join('/', grep { length && $_ ne '.' } @$name);
-		$path .= '/' if $name->[-1] eq '' || $name->[-1] eq '.';
-		$path =~ s|//+|/|g;
-		$path;
-	};
-}
-
-
-sub resolved_canonical_path {
-	my $x= $_[0]->resolve;
-	# ignore name of root entry
-	return '/'.join('/', map { $_->{dirent}->name } @$x[1..$#$x]);
-}
+sub path_name_list { @{$_[0]->path_names} }
+sub path_ent_list  { @{$_[0]->path_ents} }
+sub final_ent      { $_[0]->path_ents->[-1] }
+sub type           { $_[0]->final_ent->type }
 
 
+# methods
 sub resolve {
-	# See if we can re-use the previous result...
-	my $nodes= $_[0]{_node_path};
-	return $nodes if $nodes && ref $nodes->[-1] && !$nodes->[-1]{invalid};
-	# Only re-resolve the nodes which have been invalidated.
-	# This is part of an optimization to create half-resolved path objects.
-	my ($self, $flags)= @_;
-	my (@valid_nodes, $sub_path);
-	if ($nodes) {
-		for (@$nodes) {
-			last if !ref $_ || $_->{invalid};
-			push @valid_nodes, $_;
-		}
-		$sub_path= [ map { ref $_? $_->{dirent}->name : $_ } @{$nodes}[ scalar(@valid_nodes) .. $#$nodes ] ];
-	} else {
-		$sub_path= $self->{path_names};
-	}
-	$flags= { follow_symlinks => 1, $flags? %$flags : () };
-	$nodes= $self->{filesystem}->_resolve_path(\@valid_nodes, $sub_path, $flags);
-	if (ref $nodes) {
-		return ($self->{_node_path}= $nodes);
-	} else {
-		# else, got an error...
-		${$flags->{error_out}}= $nodes
-			if ref $flags->{error_out};
-		Carp::croak $nodes unless $flags->{no_die};
-		return undef;
-	}
+	$_[0]{path_ents}= $_[0]{filesystem}->resolve_path($_[0]{path_names})
 }
 
-
 sub path {
 	my $self= shift;
-	my @sub_names= map { File::Spec->splitdir($_) } @_;
 	bless {
-		filesystem => $self->{filesystem},
-		path_names => [ @{$self->{path_names}}, @sub_names ],
-		$self->{_node_path}? (_node_path => [ @{$self->{_node_path}}, @sub_names ]) : (),
-	}, ref $self;
-}
-
-sub path_if_exists {
-	my $self= shift;
-	my $path= $self->path(@_);
-	$path->resolve({no_die => 1})? $path : undef;
-}
-
-
-sub mkdir {
-	my $self= shift;
-	$self->{filesystem}->mkdir($self->path_names, @_);
-	$self;
+		filesystem => $self->filesystem,
+		path_names => [ @{$self->path_names}, @_ ]
+	}, ref($self);
 }
 
 
 sub file {
-	$_[0]{file} ||= do {
-		my $ent= $_[0]->dirent;
-		$ent->type eq 'file' or Carp::croak "Path is not a file";
-		defined (my $hash= $ent->ref) or Carp::croak "File was not stored in CAS";
-		$_[0]->filesystem->get($hash);
-	};
+	defined(my $hash= $_[0]->final_ent->ref)
+		or croak "Path is not a file";
+	$_[0]->filesystem->get($hash);
 }
 
 sub open {
 	$_[0]->file->open
 }
 
-
-sub dir {
-	$_[0]{dir} ||= do {
-		my $ent= $_[0]->dirent;
-		$ent->type eq 'dir' or Carp::croak "Path is not a directory";
-		defined (my $hash= $ent->ref) or Carp::croak "Directory was not stored in CAS";
-		$_[0]->filesystem->get_dir($hash);
-	}
-}
-
-
-sub readdir {
-	$_[0]{filesystem}->readdir($_[0]->path_names)
-}
-
-
-sub tree_iterator {
-	my $self= shift;
-	my %p= (@_ == 1 && ref $_[0] eq 'HASH')? %{$_[0]} : @_;
-	$self->filesystem->tree_iterator(%p, path => $self->path_names);
-}
-
-package DataStore::CAS::FS::TreeIterator;
-use strict;
-use warnings;
-use Carp;
-
-
-sub _fields { $_[0]->($_[0]) }
-
-sub new {
-	my $class= shift;
-	my $self= bless { (@_ == 1 && ref $_[0] eq 'HASH')? %{$_[0]} : @_ }, $class;
-	defined $self->{$_} || croak "'$_' is required"
-		for qw( path fs );
-	$self->{path_nodes}= \my @nodes;
-	$self->{dirstack}= \my @dirstack;
-	$self->{names}= \my @names;
-	$self->{filterref}= \my $filter;
-	$filter= delete $self->{filter};
-	my $fs= $self->{fs};
-	$self->_init;
-	return bless sub {
-		return $self if @_ && ref($_[0]) eq $class;
-		return undef unless @dirstack;
-		# back out of a directory hierarchy that we have finished
-		while (!@{$dirstack[-1]}) {
-			pop @dirstack; # back out of directory
-			pop @nodes;
-			pop @names;
-			return undef unless @dirstack;
-		}
-		# Iterate path leaf, by removing last leaf, and resolving using the
-		#  next name.
-		pop @nodes;
-		$names[-1]= shift @{$dirstack[-1]};
-		$fs->_resolve_path(\@nodes, [ $names[-1] ]);
-		my $p= bless {
-				path_names => \@names,
-				path_dirents  => \@nodes,
-				filesystem => $fs,
-			}, 'DataStore::CAS::FS::Path';
-		# If a dir, push it onto the stack
-		if ($p->type eq 'dir') {
-			push @dirstack, [ map { $_->name } @{ $fs->_get_dir_entries($nodes[-1]) } ];
-			push @nodes, undef;
-			push @names, undef;
-		}
-		return $p;
-	}, $class;
-}
-
-sub _init {
-	my $self= shift;
-	# _resolve_path returns a string on failure
-	my $x;
-	ref( $x= $self->{fs}->_resolve_path(undef, $self->{path}) )
-		or croak $x;
-	# maintain an array of resolved path nodes, and an array of
-	#  arrays of names-to-iterate for each directory
-	@{$self->{path_nodes}}= @$x;
-	@{$self->{names}}= map { $_->{dirent}->name } @$x;
-	@{$self->{dirstack}}= ([]) x @$x;
-	push @{$self->{dirstack}[-1]}, $self->{names}[-1];
-}
-
-sub reset {
-	$_[0]->($_[0])->_init;
-	1;
-}
-
-sub skip_dir {
-	my $self= $_[0]->($_[0]);
-	@{$self->{dirstack}[-1]}= ()
-		if @{$self->{dirstack}};
-	1;
-}
-
 package DataStore::CAS::FS::DirCache;
 use strict;
 use warnings;
@@ -820,7 +524,7 @@ DataStore::CAS::FS - Virtual Filesystem backed by Content-Addressable Storage
 
 =head1 VERSION
 
-version 0.010100_04
+version 0.0100
 
 =head1 SYNOPSIS
 
@@ -834,11 +538,11 @@ version 0.010100_04
   );
   
   # Open an existing root directory on an existing store
-  $casfs= DataStore::CAS::FS->new( store => $cas, root => $digest_hash );
+  $casfs= DataStore::CAS::FS->new( store => $cas, root_dir => $digest_hash );
   
   # --- These pass through to the $cas module
   
-  $hash= $casfs->put("Blah");
+  $hash= $casfs->put("Blah"); 
   $hash= $casfs->put_file("./foo/bar/baz");
   $file= $casfs->get($hash);
   
@@ -853,7 +557,7 @@ version 0.010100_04
 
 =head1 DESCRIPTION
 
-DataStore::CAS::FS extends the L<DataStore::CAS> API to support directory
+DataStore::CAS::FS extends the content-addressable API to support directory
 objects which let you store store traditional file hierarchies in the CAS,
 and look up files by a path name (so long as you know the hash of the root).
 
@@ -947,11 +651,11 @@ Parameters:
 
 =over
 
-=item C<store> - required
+=item store - required
 
 An instance of (a subclass of) L<DataStore::CAS>
 
-=item C<root_entry> - required
+=item root_entry - required
 
 An instance of L<DataStore::CAS::FS::DirEnt>, or a hashref of DirEnt fields,
 or an empty hash if you want to start from an empty filesystem, or a
@@ -959,13 +663,13 @@ L<DataStore::CAS::FS::Dir> which you want to be the root directory
 (or a L<DataStore::CAS::File> object that contains a serialized Dir) or
 or a digest hash of that File within the store.
 
-=item C<root> - alias for root_entry
+=item root - alias for root_entry
 
 =back
 
 =head2 get
 
-Alias for L<store-E<gt>get|DataStore::CAS/get>
+Alias for L<< store-E<gt>get|DataStore::CAS/get >>
 
 =head2 get_dir
 
@@ -984,23 +688,23 @@ occurs while decoding one that exists.
 
 =head2 put
 
-Alias for L<store-E<gt>put|DataStore::CAS/put>
+Alias for L<< store-E<gt>put | DataStore::CAS/put >>
 
 =head2 put_scalar
 
-Alias for L<store-E<gt>put_scalar|DataStore::CAS/put_scalar>
+Alias for L<< store-E<gt>put_scalar | DataStore::CAS/put_scalar >>
 
 =head2 put_file
 
-Alias for L<store-E<gt>put_file|DataStore::CAS/put_file>
+Alias for L<< store-E<gt>put_file | DataStore::CAS/put_file >>
 
 =head2 put_handle
 
-Alias for L<store-E<gt>put_handle|DataStore::CAS/put_handle>
+Alias for L<< store-E<gt>put_handle | DataStore::CAS/put_handle >>
 
 =head2 validate
 
-Alias for L<store-E<gt>validate|DataStore::CAS/validate>
+Alias for L<< store-E<gt>validate | DataStore::CAS/validate >>
 
 =head2 path
 
@@ -1017,48 +721,31 @@ path needs to start with the volume name, which will usually be ''.  Note that
 you get this already if you take an absolute path and pass it to
 L<< File::Spec-E<gt>splitdir|File::Spec/splitdir >>.
 
-=head2 path_if_exists
-
-  $path= $fs->path_if_exists( @path_names )
-
-This method is like L</path>, but it immediately resolves the path and returns
-undef if the path doesn't exist.  It returns the L<path object|/"PATH OBJECTS">
-if it does.
-
-=head2 tree_iterator
-
-  $iter= $fs->tree_iterator( %optional_flags )
-
-With no flags, creates a L<tree-iterator|/"TREE ITERATORS"> which iterates the
-entire tree depth-first, listing directories before their contents.
-
-When the C<path> flag is given, iterates the named path and everything
-under it (if it is a directory), or croaks if it doesn't exist.
-
 =head2 resolve_path
 
   $path_array= $fs->resolve_path( \@path_names, \%optional_flags )
   $path_array= $fs->resolve_path( $path_string, \%optional_flags )
 
-Returns an arrayref of L<DirEnt|DataStore::CAS::FS::DirEnt> objects
-corresponding to the canonical absolute specified path, starting with the
-C<root_entry>.
+Returns an arrayref of L<DataStore::CAS::FS::DirEnt> objects corresponding
+to the canonical absolute specified path, starting with the C<root_entry>.
 
-@path_names may contain empty strings C<"">, which are ignored.  This provides
-compatibility with L<File::Spec-E<gt>splitdir|File::Spec/splitdir>, and also
-with calling C<split /\//> on strings with "//" in them.
+First, a note on @path_names: you need to specify the volume, which for UNIX
+is the empty string ''.  While volumes might seem like an unnecessary
+concept, and I wasn't originally going to include that in my design, it helped
+in 2 major ways: it allows us to store a regular ::DirEnt for the root
+directory (which is useful for things like permissions and timestamp) and
+allows us to record general metadata for the filesystem as a whole, within the
+->metadata of the volume_dir.  As a side benefit, Windows users might
+appreciate being able to save backups of multiple volumes in a way that
+preserves their view of the system.  As another side benefit, it is compatible
+with L<< File::Spec-E<gt>splitdir|File::Spec/splitdir >>.
 
-If a C<$path_string> is given instead of an arrayref, it will be split by
-L<File::Spec-E<gt>splitdir|File::Spec/splitdir>, which may or may not be what
-you want.
-
-When resolving symlinks, this function operates much the same way Linux does.
-If the path you specify ends with a symlink, the result will be a DirEnt
-describing the symlink.  If the path you specify ends with a symlink and a
-C<""> (equivalent of ending with a C<'/'> or C<'/.'>), the symlink will be
-resolved to a L<DirEnt|DataStore::CAS::FS::DirEnt> for the target file or
-directory. (and if the target doesn't exist, it throws an exception, but see
-C<nodie>)
+Next, a note on resolving paths: This function will follow symlinks in much
+the same way Linux does.  If the path you specify ends with a symlink, the
+result will be a DirEnt describing the symlink.  If the path you specify
+ends with a symlink and a "" (equivalent of ending with a '/'), the symlink
+will be resolved to a DirEnt for the target file or directory. (and if
+it doesn't exist, you get an error)
 
 Also, its worth noting that the directory objects in DataStore::CAS::FS are
 strictly a tree, with no back-reference to the parent directory.  So, ".."
@@ -1072,28 +759,28 @@ the entire path, since you can't take a final directory and trace it backwards.
 
 If the path does not exist, or cannot be resolved for some reason, this method
 will either return undef or die, based on whether you provided the optional
-C<nodie> flag.
+'nodie' flag.
 
 Flags:
 
 =over
 
-=item C<no_die => $bool>
+=item no_die => $bool
 
 Return undef instead of dying
 
-=item C<error_out => \$err_variable>
+=item error_out => \$err_variable
 
 If set to a scalar-ref, the scalar ref will receive the error message, if any.
 You probably want to set 'nodie' as well.
 
-=item C<partial => $bool>
+=item partial => $bool
 
 If the path doesn't exist, any missing directories will be given placeholder
 DirEnt objects.  You can test whether the path was resolved completely by
 checking whether $result->[-1]->type is defined.
 
-=item C<mkdir => 1 || 2>
+=item mkdir => 1 || 2
 
 If mkdir is 1, missing directories will be created on demand.
 
@@ -1101,30 +788,10 @@ If mkdir is 2,
 
 =back
 
-=head2 get_dir_entries
-
-  $dirent_array= $fs->get_dir_entries( \@path )
-
-Returns an array of directory entries for the specified path.
-
-This differs from C<$fs-E<gt>path(@path)-E<gt>dir-E<gt>iterator> in that you
-see any changes that have been made via calls to C<set_path> or C<apply_path>.
-Calling C<iterator> on the directory object will only return what was recorded
-in the CAS.
-
-=head2 readdir
-
-  $names= $fs->readdir( \@path ); # returns arrayref in scalar context
-  @names= $fs->readdir( \@path ); # returns list in list context
-
-Convenience method for L</get_dir_entries>, but returns an arrayref of
-I<names> (rather than L<DirEnt/DataStore::CAS::FS::DirEnt> objects) and
-returns a list when called in list context.
-
 =head2 set_path
 
   $fs->set_path( \@path, $Dir_Entry \%optional_flags )
-  # returns 1, or dies
+  # always returns '1'
 
 Temporarily override a directory entry at @path.  If $Dir_Entry is false, this
 will cause @path to be unlinked.  If the name of Dir_Entry differs from the
@@ -1165,8 +832,8 @@ conceptually equivalent to C<< $fs= DataStore::CAS::FS->new( $fs->root_entry ) >
 
 =head2 update_path
 
-  $fs->update_path( \@path, $changes, \%optional_flags )
-  # returns 1, or dies
+  $fs->update_path( \@path, \%changes, \%optional_flags )
+  $fs->update_path( \@path, \@changes, \%optional_flags )
 
 Like L</set_path>, but it applies a hashref (or arrayref) of $changes to the
 directory entry which exists at the named path.  Use this to update a few
@@ -1178,8 +845,6 @@ attributes of a directory entry without overwriting the entire thing.
 
 Convenience method to create an empty directory at C<path>.
 
-Croaks if the path already exists and is not a directory.
-
 =head2 touch
 
   $fs->touch( \@path )
@@ -1219,80 +884,41 @@ tree.
 
 =head1 PATH OBJECTS
 
-Path objects are a simple wrapper around a path name array.  Path objects are
-lazily resolved to Filesystem Nodes.  This means the path object can exist
-even if the node does not, and a path object to "/foo" will continue to be
-usable even if "foo" is deleted and re-created.
-
-Most methods of the path objects just pass-through to the Node object returned
-from L</resolve>.
-
 =head2 path_names
 
-Arrayref of path parts.  If you created the path object from a path string
-like "/foo/bar", path_names will contain the result of File::Spec->splitdir:
-C<[ '', 'foo', 'bar' ]>.
+Arrayref of path parts
+
+=head2 path_ents
+
+Arrayref of L<DirEnt|DataStore::CAS::FS::DirEnt> objects resolved from the
+C<path_names>.  Lazy-built, so it might C<die> when accessed.
 
 =head2 filesystem
 
 Reference to the DataStore::CAS::FS it was created by.
 
-=head2 path_dirents
-
-Returns an array of L<DirEnt|DataStore::CAS::FS::DirEnt> objects resolved from
-this path.  Throws an exception if the path does not currently exist in the
-filesystem.
-
 =head2 path_name_list
 
 Convenience list accessor for path_names arrayref
 
-=head2 path_dirent_list
+=head2 path_ent_list
 
-Convenience list accessor for path_dirents arrayref
+Convenience list accessor for path_ents arrayref
 
-=head2 dirent
+=head2 final_ent
 
-Convenience accessor for final element of path_dirents
+Convenience accessor for final element of path_ents
 
 =head2 type
 
-Convenience accessor for the C<type> field of the final element of C<path_dirents>
-
-=head2 name
-
-Convenience accessor for the C<name> field of the final element of C<path_dirents>
-
-=head2 depth
-
-Convenience accessor for the number of elements in C<path_dirents> minus one.
-The root entry has a depth of 0, C<"/a"> is a depth of one, and so on.
-
-Note that this is counting the resolved path (after symlinks), not the logical
-requested path.
-
-=head2 canonical_path
-
-Returns a unix-notation absolute path, with extra '/' and '.' removed.
-
-The path is not resolved, and may contain ".."
-
-=head2 resolved_canonical_path
-
-  $fs->path("foo","..","bar","")->resolved_canonical_path
-  # where /bar is a symlink to /baz
-  # returns "/baz"
-
-Resolves the path, and then returns a canonical unix notation for it.  The
-resolved path never ends with '/' because all symlinks have been resolved,
-and it would serve no purpose.
+Convenience accessor for the C<type> field of the final element of C<path_ents>
 
 =head2 resolve
 
   $path->resolve()
 
-Resolve the path, and cache the underlying nodes until the next time the
-Filesystem is modified.
+Call L</resolve_path> for C<path_names>, and cache the result in the
+C<path_ents> attribute.  Also returns C<path_ents>.
 
 =head2 path
 
@@ -1300,93 +926,18 @@ Filesystem is modified.
 
 Get a sub-path from this path.  Returns another Path object.
 
-=head2 path_if_exists
-
-  $path->path_if_exists( \@sub_path )
-
-Returns a path object if the subpath exists.  Returns undef if not.
-
-=head2 mkdir
-
-Creates a directory at this path, possibly creating parent directories as
-well.  Dies if the path passes through an existing DirEnt which is not a
-directory.
-
 =head2 file
 
   $file= $path->file();
 
-Returns the DataStore::CAS::File of the final element of C<path_dirents>,
+Returns the DataStore::CAS::File of the final element of C<path_ents>,
 or dies trying.
 
 =head2 open
 
   $handle= $path->open
 
-Alias for C<< $path-E<gt>file-E<gt>open >>
-
-=head2 dir
-
-  $dir= $path->dir
-
-Convenience method for calling L</get_dir> on the file referred to by this
-path.  Dies if this path does not reference any content, or if it is not
-a directory.
-
-Note that this directory object is immutable, from the CAS, and will not
-reflect any changes to the filesystem until C<$fs-E<gt>commit> is called.
-
-=head2 readdir
-
-  $names= $path->readdir(); # returns arrayref in scalar context
-  @names= $path->readdir(); # returns list in list context
-
-Convenience method for C<$fs-E<gt>readdir($path->path_names)
-
-=head2 tree_iterator
-
-  $iter= $path->tree_iterator( %optional_flags )
-
-Convenience method for
-
-  $path->filesystem->tree_iterator( path => $path->path_names, %flags ).
-
-See L</tree_iterator>.
-
-=head1 TREE ITERATORS
-
-The tree iterators returned by $fs->tree_iterator and $path->iterator run a
-depth-first alphabetical pre-order traversal of the tree.  They act as
-coderefs (taking no arguments) and return a Path object, or undef at the
-end of the iteration.
-
-  while (my $path= $iter->()) {
-    print $path->resolved_path_str, "\n"
-		if $path->type ne 'dir';
-  }
-
-The iterators are also blessed objects, and have a few useful methods:
-
-=head2 reset
-
-  $iter->reset();
-
-Start iteration over from the beginning.
-
-=head2 skip_dir
-
-  $iter->skip_dir();
-
-C<skip_dir> immediately ends the current subdirectory, and the next call to
-the iterator will return the next item from the parent directory.
-
-The iterator "begins" a directory right before it returns it to you.  So, you
-can prevent entering a directory like this:
-
-  my $path= $iter->();
-  if ($path->type eq 'dir' && !we_want_to_enter_dir($path)) {
-    $iter->skip_dir;
-  }
+Alias for C<< $path->file->open >>
 
 =head1 DIRECTORY CACHE
 
@@ -1447,7 +998,7 @@ and some backward-compatible APIs exist which can represent the Unicode as
 Latin1 or whatnot on a best-effort basis.  I think the "Unicode everywhere"
 philosophy is arguably a better way to go, but as this tool is primarily
 designed with Unix in mind, and since it is intended for saving backups of real
-filesystems, it needs to be able to accurately store exactly what it finds in
+filesystems, it needs to be able to accurately store exactly what it find in
 the filesystem.  Essentially this means it neeeds to be *able* to store
 invalid UTF-8 sequences, -or- encode the octets as unicode codepoints up to
 0xFF, and later know to write them out to the filesystem as octets instead
@@ -1541,9 +1092,8 @@ when decoding the directory.
 
 =head1 SEE ALSO
 
-L<Brackup> - A similar-minded backup utility written in Perl, but without the
+C<Brackup> - A similar-minded backup utility written in Perl, but without the
 separation between library and application and with limited FUSE performance.
-(and rather sparse documentation)
 
 L<http://git-scm.com> - The world-famous version control tool
 
