@@ -4,17 +4,17 @@ use warnings;
 use Carp;
 use overload '""' => \&to_string, 'cmp' => \&str_compare, '.' => \&str_concat;
 
-our $VERSION= '0.011000';
+our $VERSION= 0.0100;
 
 # ABSTRACT: Wrapper to represent non-utf8 data in a unicode context
 
 
 sub decode_utf8 {
-	my $str= $_[-1];
-	!ref $str || ref($str)->isa(__PACKAGE__)
+	my ($class, $str)= @_;
+	!ref $str || ref($str)->isa($class)
 		or croak "Can't convert ".ref($str);
 	return ref($str) || utf8::is_utf8($str) || utf8::decode($str)? $str
-		: bless(\$str, __PACKAGE__);
+		: bless(\$str, $class);
 }
 
 sub is_non_unicode { 1 }
@@ -55,10 +55,6 @@ sub FROM_JSON {
 	return bless \$x, __PACKAGE__;
 }
 
-sub TO_UTF8 {
-	${$_[0]};
-}
-
 1;
 
 __END__
@@ -71,7 +67,7 @@ DataStore::CAS::FS::InvalidUTF8 - Wrapper to represent non-utf8 data in a unicod
 
 =head1 VERSION
 
-version 0.010100_04
+version 0.0100
 
 =head1 SYNOPSIS
 
